import os
import json
import fnmatch
import requests
import pathlib
import urllib.parse


class RunEnv(object):

    BEARER_TOKEN = os.environ.get('BEARER_TOKEN')
    APP_KEY = os.environ.get('APP_KEY')
    POLICY_SERVICE_URL = None
    DATA_PARTITION = os.environ.get('DATA_PARTITION')
    POLICIES_FOLDER = 'default-policies'
    BOOTSTRAP_SEQUENCE = 'bootstrap_sequence.json'

    def __init__(self):
        """Empty constructor"""
        pass

    def is_ok(self, policy_service=True):
        message = ''
        ok = True
        if policy_service: ok = ok and self.POLICY_SERVICE_URL is not None
        ok = ok and self.BEARER_TOKEN is not None  and self.DATA_PARTITION is not None
        message = self.__create_message(message, ok)
        return ok, message

    def __create_message(self, message, ok):
        if not ok:
            parts = list()
            if self.BEARER_TOKEN is None:
                parts.append('BEARER TOKEN')
            if self.POLICY_SERVICE_URL is None:
                parts.append('Policy service URL')
            if self.DATA_PARTITION is None:
                parts.append('Data-partition')
            message = ', '.join(parts) + ' missing.'
        return message

    @staticmethod
    def get_headers():
        return {
            'data-partition-id': RunEnv.DATA_PARTITION,
            'Content-Type': 'application/json',
            'AppKey': RunEnv.APP_KEY,
            'Authorization': RunEnv.BEARER_TOKEN
        }


class Utility(object):
    def __init__(self):
        """Empty constructor"""
        pass

    @staticmethod
    def find_file(file_name, directory_parts=None, root=os.path.abspath(__file__)):
        """Find a file with a given name in optional sub-path components and root"""
        if directory_parts is None:
            directory_parts = []
        path = Utility.__get_root_path(root)
        for part in directory_parts:
            path = os.path.join(path, part)
        for root, dirs, files in os.walk(path):
            for one_file in fnmatch.filter(files, file_name):
                return os.path.join(root, one_file)
        return None

    @staticmethod
    def find_files(directory_parts=None, root=os.path.abspath(__file__), search_expression='*.json'):
        """Find all Rego files in optional sub-path components and root"""
        found = list()
        if directory_parts is None:
            directory_parts = []
        path = Utility.__get_root_path(root)
        for part in directory_parts:
            path = os.path.join(path, part)
        for root, dirs, files in os.walk(path):
            for one_file in fnmatch.filter(files, search_expression):
                found.append(os.path.join(root, one_file))
        return found

    @staticmethod
    def __get_root_path(root):
        if os.path.isfile(root):
            path = os.path.join(os.path.dirname(root))
        else:
            path = root
        return path

    @staticmethod
    def load_json(path):
        """Load a JSON file"""
        try:
            with open(path, "r", encoding='utf-8') as text_file:
                j_obj = json.load(text_file)
            return j_obj
        except FileNotFoundError as e:
            exit("Given File path not found::{}".format(str(e)))

    @staticmethod
    def load_rego(path):
        """Load a Rego file"""
        try:
            with open(path, "r", encoding='utf-8') as text_file:
                content = text_file.read()
            return content
        except FileNotFoundError as e:
            exit("Given File path not found::{}".format(str(e)))

    @staticmethod
    def get_relative_path(base_path, path):
        return pathlib.Path(os.path.relpath(path, base_path)).as_posix()

    @staticmethod
    def path_to_deployments():
        return os.path.dirname(os.path.dirname(__file__))


class PolicyService(object):
    def __init__(self, url: str):
        self.headers = RunEnv.get_headers()
        self.url = url

    def does_policy_exist(self, id):
        policy = None
        url = '{}/{}'.format(self.url, id)
        response = requests.request("GET", url, headers=self.headers)
        if response.status_code in range(200,300):
            policy = json.loads(response.text)
        return response.status_code in range(200,300) and policy.get('code', None) != "resource_not_found", policy

    def put_policy(self, id: str, policy: str, info: list, errors: list):
        # attempt to load this kind
        exists, dummy = self.does_policy_exist(id)
        url = '{}/{}'.format(self.url, id)
        if exists:
            info.append('Policy {} already exists and was not updated.'.format(id))
            response = None
        else:
            response = requests.request('PUT', url, headers=self.headers, data=policy)

        if response is not None:
            code = response.status_code
            if code not in range(200, 300):
                errors.append(
                    'Error with policy {}: Message: {}'.format(id, response.text))
            else:
                info.append('Policy {} successfully created.'.format(id))

